from typing import Iterable, Optional

from dateutil import tz
from jdatetime import date as jdate, datetime as jdatetime, timedelta as jtimedelta, time as jtime

_TIMEZONE = tz.gettz('Asia/Tehran')


def get_tehran_timezone():
    return _TIMEZONE


def get_tehran_next_active_day(today: Optional[jdate] = None) -> jdate:
    if not today:
        today = jdatetime.today().date()

    next_day_interval = 1
    current_weekday = today.weekday()
    if current_weekday == 4:
        next_day_interval = 3
    elif current_weekday == 5:
        next_day_interval = 2

    return (get_tehran_end_of_day(today) + jtimedelta(days=next_day_interval)).date()


def get_tehran_prev_active_day(today: Optional[jdate] = None) -> jdate:
    if not today:
        today = jdatetime.today().date()

    prev_day_interval = 1
    current_weekday = today.weekday()
    if current_weekday == 5:
        prev_day_interval = 1
    elif current_weekday == 6:
        prev_day_interval = 2
    elif current_weekday == 0:
        prev_day_interval = 3

    return (get_tehran_end_of_day(today) - jtimedelta(days=prev_day_interval)).date()


def get_tehran_end_of_day(d: jdate) -> jdatetime:
    return jdatetime.combine(d, jtime(23, 59, 59, tzinfo=get_tehran_timezone()))


def jalali_date_range(start_date: jdate, end_date: jdate) -> Iterable[jdate]:
    start_time = get_tehran_end_of_day(start_date)
    end_time = get_tehran_end_of_day(end_date)

    for n in range(int((end_time - start_time).days)):
        yield (start_time + jtimedelta(n)).date()



import pandas as pd
import requests

from .config import BOURSIKA_API_BASE_URL
from .errors import ActionError


class BaseProvider:
    def __init__(self, api_key: str):
        self.api_key = api_key

    def _call_provider_action(self, service_slug: str, action_name: str, parameters: dict):
        endpoint = f'{BOURSIKA_API_BASE_URL}/providers/{service_slug}/actions/{action_name}/calls/'

        try:
            response = requests.post(endpoint, json={
                'parameters': parameters,
            }, headers={
                'ApiKey': self.api_key,
            }, timeout=15)

            response.raise_for_status()

            return response.json().get('result')
        except requests.exceptions.HTTPError as e:
            raise ActionError(message=e.response.json().get('message'))


class TseProvider(BaseProvider):
    def get_all_symbols(self) -> pd.DataFrame:
        raw = super()._call_provider_action(
            service_slug='tse',
            action_name='get_all_symbols',
            parameters={},
        )

        return pd.DataFrame(raw)

    def get_daily_ticks(self, symbol_id: str, page: int = 1) -> pd.DataFrame:
        raw = super()._call_provider_action(
            service_slug='tse',
            action_name='get_daily_ticks',
            parameters={
                'symbol_id': symbol_id,
                'page': page,
            },
        )

        return pd.DataFrame(raw)

    def get_last_daily_tick(self, symbol_id: str) -> pd.Series:
        raw = super()._call_provider_action(
            service_slug='tse',
            action_name='get_last_daily_tick',
            parameters={
                'symbol_id': symbol_id,
            },
        )

        return pd.Series(raw)



import requests

from .config import BOURSIKA_API_BASE_URL
from .enums import Trends
from .errors import ActionError
from .providers import TseProvider
from .utils.date_helpers import get_tehran_end_of_day, get_tehran_next_active_day


class BasePredictor:
    def __init__(self, api_key: str):
        self.api_key = api_key

        self.tse_provider = TseProvider(api_key=api_key)

    def _predict(self, target_key: str, target_timestamp: int, payload: dict):
        endpoint = f'{BOURSIKA_API_BASE_URL}/predictors/-/predictions/'

        try:
            response = requests.post(endpoint, json={
                'target_key': target_key,
                'target_timestamp': target_timestamp,
                'payload': payload,
            }, headers={
                'ApiKey': self.api_key,
            }, timeout=15)

            response.raise_for_status()

            return response.json().get('prediction')
        except requests.exceptions.HTTPError as e:
            raise ActionError(message=e.response.json().get('message'))


class TseDailyPricePredictorClient(BasePredictor):
    def predict(self, symbol_id: str, trend: Trends):
        target_timestamp = int(get_tehran_end_of_day(get_tehran_next_active_day()).timestamp())
        return super()._predict(
            target_key=symbol_id,
            target_timestamp=target_timestamp,
            payload={
                'trend': trend.value,
            },
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['boursika', 'boursika.utils']

package_data = \
{'': ['*']}

install_requires = \
['jdatetime>=3.6.2,<4.0.0',
 'pandas>=1.3.0,<2.0.0',
 'python-dateutil>=2.8.2,<3.0.0',
 'requests>=2.26.0,<3.0.0',
 'websocket-client>=1.1.0,<2.0.0']

setup_kwargs = {
    'name': 'boursika',
    'version': '1.0.0',
    'description': 'client for boursika-engine platform',
    'long_description': None,
    'author': 'Mahdi Sadeghi',
    'author_email': 'mail2mahsad@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'http://docs.boursika-engine.ir',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
